﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using BinaryMark.Batch.Core;

namespace CustomActionsCS
{

/// <summary>
/// This is a custom action class that defines action attributes and main processing function. As such it must implement both iProcessingStep because it is a step in the processing logic, and iAction because it is an action (as opposed to being a constraint/condition).
/// </summary>
/// <remarks>Please follow the layout of this class when designing your own actions. All members are required and must have identical signatures. You may however define additional members to suit your needs.</remarks>
public class aCycleBytes : iProcessingStep, iAction
{

	/// <summary>
	/// Local variable that holds action-specific options for this instance of the action.
	/// </summary>
	/// <remarks></remarks>
	private aCycleBytesOptions _opt;
	/// <summary>
	/// Holds the UserControl that exposes action-specific options for modification via UI. Note that it is shared by all instances of the action of this type!
	/// </summary>
	/// <remarks></remarks>

	static aCycleBytesUI _opg;
	/// <summary>
	/// Default constructor creates both the action and its options.
	/// </summary>
	/// <remarks></remarks>
	public aCycleBytes()
	{
		_opt = new aCycleBytesOptions();
		_opt.OptionsChanged += RaiseOptionsChanged;
	}

	/// <summary>
	/// This constructor must be present in exactly this form to support the infrastructure. No code is allowed inside.
	/// </summary>
	/// <param name="aVirtualized"></param>
	/// <remarks></remarks>

    public aCycleBytes(bool aVirtualized)
	{
	}

	/// <summary>
	/// Creates an instance of the action without creating its respective Options class and other objects.
	/// </summary>
	public static new iProcessingStep GetVirtualInstance()
	{
		return new aCycleBytes(true);
	}

	/// <summary>
	/// Helper routine for raising OptionsChanged event.
	/// </summary>
    public void RaiseOptionsChanged(ProcessingStepOptionsBase aSender, string AffectedPropertyName)
	{
		if (this.OptionsChanged != null) {
			this.OptionsChanged(this, string.Empty);
		}
	}

	/// <summary>
	/// Unique action ID. Note it is recommended you prefix it with some string to avoid possible collisions with other actions.
	/// </summary>
    string iProcessingStep.ID
    {
		get { return "custom_cyclebytes"; }
	}

	/// <summary>
	/// Human-readable full name of the action.
	/// </summary>
    string iProcessingStep.Name
    {
		get { return "Cycle Bytes SDK Sample"; }
	}

	/// <summary>
	/// Human-readable short name of the action.
	/// </summary>
	string iProcessingStep.ShortName {
		get { return ((iProcessingStep)this).Name; }
	}

	/// <summary>
	/// Description of what the action does.
	/// </summary>
	string iProcessingStep.Description {
		get { return "Increment or decrement the values of all bytes in a file by specified amount."; }
	}

	/// <summary>
	/// Short description of what the action does.
	/// </summary>
	string iProcessingStep.ShortDescription {
		get { return ((iProcessingStep)this).Description; }
	}

	/// <summary>
	/// Icon of the action. Presently not used. Not required.
	/// </summary>
	Image iProcessingStep.Picture {
		get { return null; }
	}

	/// <summary>
	/// Category to which the action should belong. Can be any short descriptive text, but it is recommended one of the built-in categories is used: Binary Tools, Text Tools, Converters.
	/// </summary>
	string iProcessingStep.Group {
		get { return "Binary Tools"; }
	}

	/// <summary>
	/// Action's color code.
	/// </summary>
	Color iProcessingStep.Color {
		get { return Color.Magenta; }
	}

	/// <summary>
	/// If the action has any options, the value should be True.
	/// </summary>
	bool iProcessingStep.HasOptions {
		get { return true; }
	}

	/// <summary>
	/// Specifies the type of the action. The only valid values are StreamAction, FileAction, and Constraint. If your action modifies file contents in any way, it should be a StreamAction; if it only performs a file operation or does not modify the file's contents, it should be a FileAction; and if it implements iConstraint, it must be Constraint.
	/// </summary>
	ProcessingStepType iProcessingStep.Type {
		get { return ProcessingStepType.StreamAction; }
	}

	/// <summary>
	/// Fires when the action-specific options have changed.
	/// </summary>
	/// <remarks>Required even if action has no options.</remarks>
    public event iProcessingStep.OptionsChangedEventHandler OptionsChanged;
    
	/// <summary>
	/// Provides access to action-specific options for this instance.
	/// </summary>
	/// <remarks>Required even if action has no options.</remarks>
	ProcessingStepOptionsBase iProcessingStep.Options {
		get { return _opt; }
		set {
			//This outline must be followed. All code is necessary if the action has options.
			_opt.OptionsChanged -= RaiseOptionsChanged;
			_opt = (aCycleBytesOptions)value;
			_opt.OptionsChanged += RaiseOptionsChanged;
			PrepareOptions();
		}
	}

	/// <summary>
	/// Helper routine that prepares the associated UserControl to display this instance's specific options.
	/// </summary>
	/// <remarks></remarks>
	private void PrepareOptions()
	{
		if (_opg == null)
			_opg = new aCycleBytesUI();
		//If the user control has not been created yet, create it. Note: it is the only place where it should be created!
		_opg.Prepare(ref _opt);
		//Prepare user control to display the options for this instance.
	}

	/// <summary>
	/// Provides access to the user control that allows modification of the current instance's options.
	/// </summary>
	UserControl iProcessingStep.OptionsUI {
		get {
			PrepareOptions();
			//This is needed so that whenever user control is requested it is always in sync with current instance's options.
			return _opg;
		}
	}

	/// <summary>
	/// This is action's primary processing routine. It performs the actual work that the action is supposed to do.
	/// </summary>
	/// <param name="input">This encapsulates a file that should be processed along with additional information about it.</param>
	/// <returns>The function should return a modified ProcessingObject (typically with it's OutputStream updated.</returns>
	ProcessingObject iAction.Process(ProcessingObject input)
	{
		//Perform basic checks
		if (input.InputStream == null)
			return input.SwapStreamsAndReturn("No Input Stream");
		if (input.InputStream.Length == 0)
			return input.SwapStreamsAndReturn("Input Stream's length is zero!");
		if (_opt.Value == 0)
			return input.SwapStreamsAndReturn("Value is set to zero, no cycling is needed!");
		//Set variables
		Stream mis = input.InputStream;
		Stream mos = input.OutputStream;
		short _val = _opt.Value;
		long ccounter = 0;
		byte[] rbuffer = new byte[100000 - 2];
		int rbuffer_cnt = 0;
		//NOTE: If you are dealing with text instead of raw data, you will need to use StreamReader and StreamWriter classes
		//NOTE: For raw/binary data you may also want to use BinaryReader and BinaryWriter classes

		//Perform the work
		do {
			rbuffer_cnt = mis.Read(rbuffer, 0, 100000);
			if (rbuffer_cnt < 1)
				break; // TODO: might not be correct. Was : Exit Do
			for (int i = 0; i <= rbuffer_cnt - 1; i++) {
				short val = (short)(rbuffer[i] + _val);
				if (val > 255) {
					val -= 255;
				} else if (val < 0) {
					val += 255;
				}
				rbuffer[i] = (byte)_val;
			}
			mos.Write(rbuffer, 0, rbuffer_cnt);
			ccounter += rbuffer_cnt;
		} while (true);

		//Flush the stream
		mos.Flush();
		//Set remaining properties
		input.WasModified = true;
		//Return result
		return input;
	}

}

/// <summary>
/// This class defined action-specific options, if your action has any options.
/// </summary>
/// <remarks>It must inherit from ProcessingStepOptionsBase</remarks>
[Serializable()]
public class aCycleBytesOptions : ProcessingStepOptionsBase
{

	//+ or - [1-255]
	private short _Value = 1;
	/// <summary>
	/// This is an example of how to properly implement one of action;s options. It must be a Public property that supports both binary and XML serialization.
	/// </summary>
	public short Value {
		get { return _Value; }
		set {
			_Value = value;
			if (!Silent) //this is needed to properly notify the program that an option was changed, so that UI can reflect the change.
            {
			    object me = this;
                base.RaiseOptionsChanged(ref me, "value");
            }
		}
	}

	/// <summary>
	/// This provides a summary of the action's options.
	/// </summary>
	/// <param name="aShort">Determines whether or not the summary should be in the short or long form. Short form should not exceed more than 150 characters.</param>
	public override string Summary(bool aShort = true) {
		 return string.Format("Cycle bytes: {0}", _Value); 
	}

}

}