Imports System.IO
Imports System.Windows.Forms
Imports System.Drawing
Imports BinaryMark.Batch.Core

''' <summary>
''' This is a custom action class that defines action attributes and main processing function. As such it must implement both iProcessingStep because it is a step in the processing logic, and iAction because it is an action (as opposed to being a constraint/condition).
''' </summary>
''' <remarks>Please follow the layout of this class when designing your own actions. All members are required and must have identical signatures. You may however define additional members to suit your needs.</remarks>
Friend Class aCycleBytes : Implements iProcessingStep, iAction

    ''' <summary>
    ''' Local variable that holds action-specific options for this instance of the action.
    ''' </summary>
    ''' <remarks></remarks>
    Private _opt As aCycleBytesOptions
    ''' <summary>
    ''' Holds the UserControl that exposes action-specific options for modification via UI. Note that it is shared by all instnaces of the action of this type!
    ''' </summary>
    ''' <remarks></remarks>
    Shared _opg As aCycleBytesUI

    ''' <summary>
    ''' Default constructor creates both the action and its options.
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        _opt = New aCycleBytesOptions
        AddHandler _opt.OptionsChanged, AddressOf RaiseOptionsChanged
    End Sub

    ''' <summary>
    ''' This constructor must be present in exactly this form to support the infrastructure. No code is allowed inside.
    ''' </summary>
    ''' <param name="aVirtualized"></param>
    ''' <remarks></remarks>
    Sub New(ByVal aVirtualized As Boolean)

    End Sub

    ''' <summary>
    ''' Creates an instance of the action without creating its respective Options class and other objects.
    ''' </summary>
    Public Shared Shadows Function GetVirtualInstance() As iProcessingStep
        Return New aCycleBytes(True)
    End Function

    ''' <summary>
    ''' Helper routine for raising OptionsChanged event.
    ''' </summary>
    Private Sub RaiseOptionsChanged(ByVal aSender As ProcessingStepOptionsBase, ByVal AffectedPropertyName As String)
        RaiseEvent OptionsChanged(Me, String.Empty)
    End Sub

    ''' <summary>
    ''' Unique action ID. Note it is recommended you prefix it with some string to avoid possible collisions with other actions.
    ''' </summary>
    Private ReadOnly Property ID As String Implements iProcessingStep.ID
        Get
            Return "custom_cyclebytes"
        End Get
    End Property

    ''' <summary>
    ''' Human-readable full name of the action.
    ''' </summary>
    Private ReadOnly Property Name As String Implements iProcessingStep.Name
        Get
            Return "Cycle Bytes SDK Sample"
        End Get
    End Property

    ''' <summary>
    ''' Human-readable short name of the action.
    ''' </summary>
    Private ReadOnly Property ShortName As String Implements iProcessingStep.ShortName
        Get
            Return Name
        End Get
    End Property

    ''' <summary>
    ''' Description of what the action does.
    ''' </summary>
    Private ReadOnly Property Description As String Implements iProcessingStep.Description
        Get
            Return "Increment or decrement the values of all bytes in a file by specified amount."
        End Get
    End Property

    ''' <summary>
    ''' Short description of what the action does.
    ''' </summary>
    Private ReadOnly Property ShortDescription As String Implements iProcessingStep.ShortDescription
        Get
            Return Description
        End Get
    End Property

    ''' <summary>
    ''' Icon of the action. Presently not used. Not required.
    ''' </summary>
    Private ReadOnly Property Picture As System.Drawing.Image Implements iProcessingStep.Picture
        Get
            Return Nothing
        End Get
    End Property

    ''' <summary>
    ''' Category to which the action should belong. Can be any short descriptive text, but it is recommended one of the built-in categories is used: Binary Tools, Text Tools, Converters.
    ''' </summary>
    Private ReadOnly Property Group As String Implements iProcessingStep.Group
        Get
            Return "Binary Tools"
        End Get
    End Property

    ''' <summary>
    ''' Action's color code.
    ''' </summary>
    Private ReadOnly Property Color As Color Implements iProcessingStep.Color
        Get
            Return Drawing.Color.Magenta
        End Get
    End Property

    ''' <summary>
    ''' If the action has any options, the value should be True.
    ''' </summary>
    Private ReadOnly Property HasOptions As Boolean Implements iProcessingStep.HasOptions
        Get
            Return True
        End Get
    End Property

    ''' <summary>
    ''' Specifies the type of the action. The only valid values are StreamAction, FileAction, and Constraint. If your action modifies file contents in any way, it should be a StreamAction; if it only performs a file operation or does not modify the file's contents, it should be a FileAction; and if it implements iConstraint, it must be Constraint.
    ''' </summary>
    Private ReadOnly Property Type As ProcessingStepType Implements iProcessingStep.Type
        Get
            Return ProcessingStepType.StreamAction
        End Get
    End Property

    ''' <summary>
    ''' Fires when the action-specific options have changed.
    ''' </summary>
    ''' <remarks>Required even if action has no options.</remarks>
    Private Event OptionsChanged(ByVal sender As iProcessingStep, ByVal AffectedPropertyName As String) Implements iProcessingStep.OptionsChanged

    ''' <summary>
    ''' Provides access to action-specific options for this instance.
    ''' </summary>
    ''' <remarks>Required even if action has no options.</remarks>
    Private Property Options As ProcessingStepOptionsBase Implements iProcessingStep.Options
        Get
            Return _opt
        End Get
        Set(ByVal value As ProcessingStepOptionsBase)
            'This outline must be followed. All code is necessary if the action has options.
            RemoveHandler _opt.OptionsChanged, AddressOf RaiseOptionsChanged
            _opt = value
            AddHandler _opt.OptionsChanged, AddressOf RaiseOptionsChanged
            PrepareOptions()
        End Set
    End Property

    ''' <summary>
    ''' Helper routine that prepares the associated UserControl to display this instance's specific options.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PrepareOptions()
        If _opg Is Nothing Then _opg = New aCycleBytesUI 'If the user control has not been created yet, create it. Note: it is the only place where it should be created!
        _opg.Prepare(_opt) 'Prepare user control to display the options for this instance.
    End Sub

    ''' <summary>
    ''' Provides access to the user control that allows modification of the current instance's options.
    ''' </summary>
    Private ReadOnly Property OptionsPanel As UserControl Implements iProcessingStep.OptionsUI
        Get
            PrepareOptions() 'This is needed so that whenever user control is requested it is always in sync with current instance's options.
            Return _opg
        End Get
    End Property

    ''' <summary>
    ''' This is action's primary processing routine. It performs the actual work that the action is supposed to do.
    ''' </summary>
    ''' <param name="input">This encapsulates a file that should be processed along with additional information about it.</param>
    ''' <returns>The function should return a modified ProcessingObject (typically with it's OutputStream updated.</returns>
    Private Function Process(ByVal input As ProcessingObject) As ProcessingObject Implements iAction.Process
        'Perform basic checks
        If input.InputStream Is Nothing Then Return input.SwapStreamsAndReturn("No Input Stream")
        If input.InputStream.Length = 0 Then Return input.SwapStreamsAndReturn("Input Stream's length is zero!")
        If _opt.Value = 0 Then Return input.SwapStreamsAndReturn("Value is set to zero, no cycling is needed!")
        'Set variables
        Dim mis As Stream = input.InputStream
        Dim mos As Stream = input.OutputStream
        Dim _val As Short = _opt.Value
        Dim ccounter As Long = 0
        Dim rbuffer(100000 - 1) As Byte
        Dim rbuffer_cnt As Integer = 0
        'NOTE: If you are dealing with text instead of raw data, you will need to use StreamReader and StreamWriter classes
        'NOTE: For raw/binary data you may also want to use BinaryReader and BinaryWriter classes

        'Perform the work
        Do
            rbuffer_cnt = mis.Read(rbuffer, 0, 100000)
            If rbuffer_cnt < 1 Then Exit Do
            For i = 0 To rbuffer_cnt - 1
                Dim val As Short = CShort(rbuffer(i)) + _val
                If val > 255 Then
                    val -= 255
                ElseIf val < 0 Then
                    val += 255
                End If
                rbuffer(i) = CByte(_val)
            Next
            mos.Write(rbuffer, 0, rbuffer_cnt)
            ccounter += rbuffer_cnt
        Loop

        'Flush the stream
        mos.Flush()
        'Set remaining properties
        input.WasModified = True
        'Return result
        Return input
    End Function

End Class

''' <summary>
''' This class defined action-specific options, if your action has any options.
''' </summary>
''' <remarks>It must inherit from ProcessingStepOptionsBase</remarks>
<Serializable()> _
Public Class aCycleBytesOptions : Inherits ProcessingStepOptionsBase

    Private _Value As Short = 1 '+ or - [1-255]
    ''' <summary>
    ''' This is an example of how to properly implement one of action;s options. It must be a Public property that supports both binary and XML serialization.
    ''' </summary>
    Public Property Value As Short
        Get
            Return _Value
        End Get
        Set(value As Short)
            _Value = value
            If Not Silent Then RaiseOptionsChanged(Me, "value") 'this is needed to properly notify the program that an option was changed, so that UI can reflect the change.
        End Set
    End Property

    ''' <summary>
    ''' This provides a summary of the action's options.
    ''' </summary>
    ''' <param name="aShort">Determines whether or not the summary should be in the short or long form. Short form should not exceed more than 150 characters.</param>
    Public Overrides ReadOnly Property Summary(Optional ByVal aShort As Boolean = True) As String
        Get
            Return String.Format("Cycle bytes: {0}", _Value)
        End Get
    End Property

End Class